class Output {

  constructor() {
    this.abapCode = `
***************************************************************
* This code was generated by openapi-json-to-abap on {{date}}.
* {{title}}: v{{version}} 
***************************************************************
    DATA: ls_meta TYPE /weco/ws_oai_meta,
    ls_metapath TYPE /weco/ws_oai_metapath,
    ls_metareqbody TYPE /weco/ws_oai_metareqbody,
    ls_metaprop TYPE /weco/ws_oai_metaprop,
    ls_metaparam TYPE /weco/ws_oai_metaparam.    
    `;

    this.pathStatement = `
    CLEAR: ls_metapath.
    ls_metapath-path = '{{path}}'.
    ls_metapath-meth = '{{method}}'.
    ls_metapath-oprid = '{{operationId}}'.
    ls_metapath-struci = '{{wecoRequest}}'.
    ls_metapath-secur = '{{security}}'.
    `;

    this.parameterStatement = `
    CLEAR: ls_metaparam.
    ls_metaparam-name = '{{name}}'.
    ls_metaparam-parin = '{{inSegment}}'.
    ls_metaparam-flreq = '{{required}}'.
    ls_metaparam-type = '{{type}}'.
    {{parameterEnumStatement}}
    APPEND ls_metaparam TO ls_metapath-param.
    `;

    this.parameterEnumStatement = `APPEND '{{enumValue}}' TO ls_metaparam-enum.
    `;

    this.requestBodyStatement = `
    CLEAR: ls_metareqbody.
    ls_metareqbody-flreq = '{{required}}'.
    ls_metareqbody-ctype = '{{contentType}}'.
    `;

    this.requestBodyPropertyStatement = `
    CLEAR: ls_metaprop.
    ls_metaprop-name = '{{name}}'.
    ls_metaprop-flreq = '{{required}}'.
    ls_metaprop-type = '{{type}}'.
    {{requestBodyPropertyEnumStatement}}    
    APPEND ls_metaprop TO ls_metareqbody-propr.
    `;

    this.requestBodyPropertyEnumStatement = `APPEND '{{enumValue}}' TO ls_metaprop-enum.
    `;    
  }

  finishPath() {
    var s = `
    APPEND ls_metapath TO rs_meta-path.\n`;

    this.appendStatement(s);
  }

  finishRequestBody() {
    var s = `
    ls_metapath-reqbo = ls_metareqbody.\n`;
    this.appendStatement(s);
  }

  addMeta(version, title) {
    var s = this.abapCode;
    s = s.replace('{{version}}', version);
    s = s.replace('{{title}}', title);

    var date = new Date(Date.now()).toISOString();
    s = s.replace('{{date}}', date);
    this.abapCode = s;
  }

  addPath(path, method, operationId, wecoRequest, security) {
    var s = this.pathStatement;
    s = s.replace('{{path}}', path);
    s = s.replace('{{method}}', method);
    s = s.replace('{{operationId}}', operationId);
    s = s.replace('{{wecoRequest}}', wecoRequest);
    s = s.replace('{{security}}', security === undefined ? '' : security);
    this.appendStatement(s);
  }

  addParameter(name, inSegment, required, type, enumValues) {
    var s = this.parameterStatement;
    s = s.replace('{{name}}', name);
    s = s.replace('{{inSegment}}', inSegment);
    s = s.replace('{{required}}', required ? 'X' : '');
    s = s.replace('{{type}}', type);

    var esTemplate = this.parameterEnumStatement;
    var enumStatement = "";
    enumValues.forEach(value => {
      var e = esTemplate.replace('{{enumValue}}', value);
      enumStatement = enumStatement + e;
    })

    s = s.replace('{{parameterEnumStatement}}', enumStatement);
    this.appendStatement(s);
  }

  addRequestBody(required, contentType) {
    var s = this.requestBodyStatement;
    s = s.replace('{{required}}', required ? 'X' : '');
    s = s.replace('{{contentType}}', contentType);
    this.appendStatement(s);
  }

  addRequestBodyProperty(name, type, required, enumValues) {
    var s = this.requestBodyPropertyStatement;
    s = s.replace('{{name}}', name);
    s = s.replace('{{type}}', type);
    s = s.replace('{{required}}', required ? 'X' : '');

    var esTemplate = this.requestBodyPropertyEnumStatement;
    var enumStatement = "";
    enumValues.forEach(value => {
      var e = esTemplate.replace('{{enumValue}}', value);
      enumStatement = enumStatement + e;
    })

    s = s.replace('{{requestBodyPropertyEnumStatement}}', enumStatement);

    this.appendStatement(s);
  }

  appendStatement(statement) {
    this.abapCode += statement;
  }

  getAbapCode() {
    var finalCode = `method GET_META_RAW.
    `
      + this.abapCode +
      `endmethod.`;

    return finalCode;
  }

}

module.exports = Output;